open Printf

let usage () =
  print_string "\
Configuration of OCamlDuceBazar

Usage: ./configure [OPTION]...

Defaults for the options are specified in brackets.

Options:
 --help                display this help and exit

Optional features:
 --with-FEATURE        force support for FEATURE [default: autodetect]
 --without-FEATURE     disable support for FEATURE [default: autodetect]

Available features:
 opt                   compile with ocamlduceopt
 byte                  compile with ocamlducec
 pxp                   support for the PXP XML parser
 expat                 support for the expat XML parser
 xmllight              support for the XML-Light parser
 json                  support for the json-wheel JSON parser

Example:

./configure --with-pxp --without-expat
"


let features = 
  [
    "opt", ref `auto;
    "byte", ref `auto;
    "pxp", ref `auto;
    "expat", ref `auto;
    "xmllight", ref `auto;
    "json", ref `auto;
  ]

(* Logging, errors, warnings *)
  
let log s =
  let oc = open_out_gen [ Open_append; Open_creat ] 0o600 "configure.log" in
  output_string oc s;
  output_char oc '\n';
  close_out oc;
  flush stdout

let fatal s = 
  let s = sprintf "*** Fatal error: %s" s in
  print_endline s; 
  log s;
  exit 1

let warning s = 
  let s = sprintf "* Warning: %s" s in
  print_endline s;
  log s

let print s = print_string s; flush stdout

(* External commands *)

let command s =
  log ("==> " ^ s);
  Sys.command (sprintf "%s >> configure.log 2>&1" s) = 0

let check_feature f p =
  printf "%s: " f;
  match !(List.assoc f features) with
    | `no -> 
	print "disabled\n"; false
    | `yes -> 
	print "checking... ";
	if p () 
	then (print "ok\n"; true) 
	else (print "failed !\n"; fatal 
	  (Printf.sprintf "Required feature %s is not available" f))
    | `auto -> 
	print "autodetecting... ";
	if p () 
	then (print "enabled\n"; true)
	else (print "disabled\n"; false)

let require_feature f p =
  printf "%s: checking... " f;
  if p () 
  then print "ok\n" 
  else (print "failed !\n"; fatal 
    (Printf.sprintf "Required feature %s is not available" f))

(* Parse command line *)

let start_with s p =
  let ls = String.length s and lp = String.length p in
  if (ls >= lp) && (String.sub s 0 lp = p)
  then Some (String.sub s lp (ls - lp)) else None
    
let parse_arg s =
  if s = "--help" then (usage (); exit 0)
  else
    match start_with s "--with-" with
      | Some f -> (List.assoc f features) := `yes
      | None ->
	  match start_with s "--without-" with
	    | Some f -> (List.assoc f features) := `no
	    | None -> raise Not_found


let () =
  print_endline "Configuring CDuce for compilation...\n";
  (try for i = 1 to Array.length Sys.argv - 1 do parse_arg Sys.argv.(i) done
  with Not_found -> usage (); fatal "Incorrect command line");
  (try Sys.remove "configure.log" with Sys_error _ -> ());
  ignore (Sys.command "uname -a >> configure.log");
  require_feature "ocamlducefind" 
    (fun () -> command "ocamlducefind printconf")

let print s = print_string s; flush stdout

let opt = 
  check_feature "opt" (fun () -> command "ocamlducefind ocamlopt")
let byte = 
  check_feature "byte" (fun () -> command "ocamlducefind ocamlc")
let () =
  if not (opt || byte) then
    fatal "One of 'opt' or 'byte' should be enabled"

let check_pkg p () =
  let comp s =
    command 
      (sprintf 
	  "ocamlducefind %s -package %s -linkpkg -o configure.try && rm -f configure.try"
	  s
	  p) 
  in
  (not opt || comp "ocamlopt") && (not byte || comp "ocamlc")

let pxp = check_feature "pxp" (check_pkg "pxp")
let expat = check_feature "expat" (check_pkg "expat")
let xmllight = check_feature "xmllight" (check_pkg "xml-light")
let json = check_feature "json" (check_pkg "json-wheel")


let () =
  if not (command "make compute_depend") then
    failwith "Error while computing dependencies";

  print "Creating Makefile.conf...\n";

  let out = open_out "Makefile.conf" in
  fprintf out "# This file has been generated by the configure script\n";
  let bool n v = fprintf out "%s=%b\n" n v in
  bool "OPT" opt;
  bool "BYTE" byte;
  bool "PXP" pxp;
  bool "EXPAT" expat;
  bool "XMLLIGHT" xmllight;
  bool "JSON" json;
  close_out out
